import 'package:go_router/go_router.dart';
import 'package:starting_template/core/extensions/go_route_guard.dart';

extension GoRouteListExtension on Iterable<GoRoute> {
  /// Returns a list of public routes.
  List<GoRoute> get publicRoutes =>
      where((route) => route.isPublic).toList(growable: false);

  /// Returns a list of protected routes.
  List<GoRoute> get protectedRoutes =>
      where((route) => route.isProtected).toList(growable: false);

  /// Returns a list of routes that are neither public nor protected.
  List<GoRoute> get noneRoutes =>
      where((route) => route.isNone).toList(growable: false);

  GoRoute fromName(String name) => firstWhere((route) => route.name == name);
  GoRoute fromPath(String path) => firstWhere((route) => route.path == path);

  GoRoute operator [](String name) => fromName(name);
}
