import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:starting_template/core/constants/constants.dart';
import 'package:starting_template/core/extensions/debuggable_bloc.dart';
import 'package:starting_template/core/logger/log_level.dart';
import 'package:starting_template/core/mixins/debuggable.dart';
import 'package:starting_template/core/mixins/loggable.dart';

class AppBlocObserver extends BlocObserver with Debuggable, Loggable {
  AppBlocObserver({
    this.printEvent = true,
    this.printError = true,
    this.printTransition = false,
    this.printChange = true,
    this.fullPrint = false,
  });

  final bool printEvent;
  final bool printError;
  final bool printChange;
  final bool printTransition;
  final bool fullPrint;

  @override
  String get debugName => 'AppBlocObserver';

  String sanitize(Object? object) {
    final message = object.toString();

    return fullPrint
        ? message
        : message.substring(
            0,
            message.length < kBlocObserverMessageLength
                ? message.length
                : kBlocObserverMessageLength,
          );
  }

  @override
  void onEvent(Bloc<dynamic, dynamic> bloc, Object? event) {
    super.onEvent(bloc, event);
    if (printEvent && bloc.showInObserver) {
      logger.log(
        LogLevel.bloc,
        '\nonEvent:        ${bloc.runtimeType}\n'
        '> event:        ${sanitize(event)}',
      );
    }
  }

  @override
  void onError(BlocBase<dynamic> bloc, Object error, StackTrace stackTrace) {
    if (printError && bloc.showInObserver) {
      logger.log(
        LogLevel.bloc,
        '\nonError:        ${bloc.runtimeType}\n'
        '> error:        ${sanitize(error)}\n'
        '$stackTrace',
      );
    }
    super.onError(bloc, error, stackTrace);
  }

  @override
  void onChange(BlocBase<dynamic> bloc, Change<dynamic> change) {
    super.onChange(bloc, change);
    if (printChange && bloc.showInObserver) {
      logger.log(
        LogLevel.bloc,
        '\nonChange:       ${bloc.runtimeType}\n'
        '> currentState: ${sanitize(change.currentState)}\n'
        '> nextState:    ${sanitize(change.nextState)}',
      );
    }
  }

  @override
  void onTransition(
    Bloc<dynamic, dynamic> bloc,
    Transition<dynamic, dynamic> transition,
  ) {
    super.onTransition(bloc, transition);
    if (printTransition && bloc.showInObserver) {
      logger.log(
        LogLevel.bloc,
        '\nonTransition:   ${bloc.runtimeType}\n'
        '> currentState: ${sanitize(transition.currentState)}\n'
        '> event:        ${sanitize(transition.event)}\n'
        '> nextState:    ${sanitize(transition.nextState)}',
      );
    }
  }
}
